/*
  Copyright 1996-2003
  Simon Whiteside

*/
package simkin.examples.dialog;
import simkin.*;
import java.io.*;
import org.xml.sax.SAXException;
import org.w3c.dom.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This class shows how you can use Simkin to show dialogs described in script
*/
public class ScriptedDialog extends XMLExecutable implements ActionListener{
    /** 
     * this class holds the name and associated method name for a component
     */
    class ControlInfo {
	String m_Name="";
	String m_Method="";
	ControlInfo(String name,String method){
	    m_Name=name;
	    m_Method=method;
	}
    }
    /**
     * the parent frame
     */
    Frame m_Frame=new Frame();
    /**
     * the actual dialog created
     */
    Dialog m_Dialog;
    /**
     * a hashtable mapping controls to ControlInfo objects
     */
    Hashtable m_ControlInfo=new Hashtable();
    /**
     * the name of the XML file
     */
    public String FileName;
    /**
     * Interpreter to use for method calls
     */
    Interpreter m_Interpreter=new Interpreter();

    public ScriptedDialog(String fileName) throws FileNotFoundException, SAXException, IOException{
	super(new FileInputStream(fileName));
	FileName=fileName;
	load();
    }
    /**
     * this method loads the dialog definition from the XML file
     */
    void load(){
	// extract the "dialog" tag from the document
	Element dialog=XMLElementObject.findChild(getElement(),"dialog");
	if (dialog!=null){
	    String title=getStringField(dialog,"title");
	    m_Dialog=new Dialog(m_Frame,title);
	    Panel dialogPanel=new Panel();
	    dialogPanel.setLayout(new FlowLayout());
	    loadControls(dialogPanel,dialog);
	    m_Dialog.add(dialogPanel);
	    int width=Integer.parseInt(getStringField(dialog,"width"));
	    int height=Integer.parseInt(getStringField(dialog,"height"));
	    m_Dialog.setSize(width,height);
	    m_Dialog.setModal(true);
	    m_Dialog.show();
	    // call the init in the XML file
	    try{
		ExecutableContext context=new ExecutableContext(m_Interpreter);
		method("init",null,context);
	    }catch(Exception e){
	    }
	}
    }
    /**
     * this method loads controls from the controls sub-element of the given element
     * @param panel the panel the controls will be added to
     * @param element the owner of the controls element
     */
    void loadControls(Panel panel,Element element){
	Element controls=XMLElementObject.findChild(element,"controls");
	if (controls!=null){
	    NodeList control_list=controls.getChildNodes();
	    for (int i=0;i<control_list.getLength();i++){
		Node node=control_list.item(i);
		if (node.getNodeName().equals("control")){
		    Element control=(Element)node;
		    String type=getStringField(control,"type");
		    String text=getStringField(control,"text");
		    String name=getStringField(control,"name");
		    String method=getStringField(control,"method");
		    Component comp=null;
		    if (type.equals("static"))
			comp=new Label(text);
		    if (type.equals("edit"))
			comp=new TextField(text,20);
		    if (type.equals("button")){
			Button button=new Button(text);
			button.addActionListener(this);
			comp=button;
		    }
		    if (type.equals("panel")){
			Panel newPanel=new Panel();
			comp=newPanel;
			// load any controls in this panel
			loadControls(newPanel,control);
		    }
		    if (comp!=null){
			panel.add(comp);
			m_ControlInfo.put(comp,new ControlInfo(name,method));
		    }
		}
	    }
	}
    }
    /**
     * this method catches button clicks and calls methods
     */
    public void actionPerformed(ActionEvent e) {
	ControlInfo info=(ControlInfo)m_ControlInfo.get(e.getSource());
	if (info!=null){
	    try{
		ExecutableContext context=new ExecutableContext(m_Interpreter);
		method(info.m_Method,null,context);
	    }catch(Exception ex){
		Tracer.trace(ex);
	    }
	}
    }
    /**
     * This method runs a command - because it is public it can be called from Simkin scripts
     */
    public void run(String command){
	try{
	    Runtime.getRuntime().exec(command);
	}catch (Exception e){
	}
    }
    /**
     * This method shows a message box - because it is public it can be called from Simkin scripts
     */
    public void msgBox(String str){
	MsgBox msgBox=new MsgBox(m_Frame,str);
	msgBox.dispose();
    }
    /**
     * this method makes the dialog reload itself from the XML stream
     */
    public void reload(){
	// recreate ourselves
	m_Dialog.hide();
	m_Dialog.dispose();
	try{
	    super.load(new FileInputStream(FileName));
	    load();
	}catch (Exception e){
	    e.printStackTrace();
	    System.exit(0);
	}
    }
    /**
     * This method closes the dialog  - because it is public it can be called from Simkin scripts
     */
    public void close(){
	m_Dialog.hide();
	m_Dialog.dispose();
	System.exit(0);
    }
    /**
     * This method sets the focus to a particular control  - because it is public it can be called from Simkin scripts
     * @param name the control name
     */
    public void setFocus(String name){
	Component comp=findControl(name);
	if (comp!=null)
	    comp.requestFocus();
    }
    /**
     * This method returns the text in a text control  - because it is public it can be called from Simkin scripts
     * @param name the control name
     * @return the text in the control (or blank if the control is not a text control)
     */
    public String getText(String name){
	String return_value="";
	Component comp=findControl(name);
	if (comp!=null){
	    if (comp instanceof TextField)
		return_value=((TextField)comp).getText();
	}
	return return_value;
    }
    /**
     * this function finds a control with the given name
     */
    Component findControl(String name){
	Enumeration keys=m_ControlInfo.keys();
	Component found=null;
	while (keys.hasMoreElements()){
	    Component comp=(Component)keys.nextElement();
	    ControlInfo info=(ControlInfo)m_ControlInfo.get(comp);
	    if (info.m_Name.equals(name)){
		found=comp;
		break;
	    }
	}
	return found;
    }
    /** 
     * this function extracts a value from the XML file as a string
     * @param parent - the owning element
     * @param fieldName - the tag name of the element within the parent containing the value
     */
    String getStringField(Element parent,String fieldName){
	return parent.getAttribute(fieldName);
    }
    /** 
     * pick up the name of the dialog file from the command line, and then create an instance of ScriptedDialog from the XML file name
     */
    static public void main(String args[]){
	if (args.length==1){
	    try{
		new ScriptedDialog(args[0]);
	    }catch(Exception e){
		e.printStackTrace();
	    }
	}else
	    System.out.println("Syntax as: java simkin.examples.ScriptedDialog <dialog file>");
    }
}
